Package: gDRimport
Type: Package
Title: Package for handling the import of dose-response data
Version: 1.11.4
Date: 2026-06-16
Authors@R: c(
    person("Arkadiusz", "Gladki", role=c("aut", "cre"), email="gladki.arkadiusz@gmail.com",
           comment = c(ORCID = "0000-0002-7059-6378")),
    person("Bartosz", "Czech", role="aut", 
           comment = c(ORCID = "0000-0002-9908-3007")),
    person("Marc", "Hafner", role="aut",
           comment = c(ORCID = "0000-0003-1337-7598")),
    person("Sergiu", "Mocanu", role="aut"),
    person("Dariusz", "Scigocki", role="aut"),
    person("Allison", "Vuong", role="aut"),
    person("Luca", "Gerosa", role="aut",
           comment = c(ORCID = "0000-0001-6805-9410")),
    person("Janina", "Smola", role="aut"))
Description: The package is a part of the gDR suite. It helps to prepare raw drug response data for downstream processing. It mainly contains helper functions for importing/loading/validating dose-response data provided in different file formats. 
License: Artistic-2.0
LazyLoad: yes
Depends: R (>= 4.2)
Imports: assertthat, BumpyMatrix, checkmate, CoreGx, PharmacoGx,
        data.table, futile.logger, gDRutils (>= 1.7.1), magrittr,
        methods, MultiAssayExperiment, readxl, rio, S4Vectors, stats,
        stringi, SummarizedExperiment, tibble, tools, utils, XML, yaml,
        openxlsx, qs2
Suggests: BiocStyle, gDRtestData (>= 1.7.1), gDRstyle (>= 1.7.1),
        knitr, purrr, testthat
URL: https://github.com/gdrplatform/gDRimport,
        https://gdrplatform.github.io/gDRimport/
BugReports: https://github.com/gdrplatform/gDRimport/issues
biocViews: Software, Infrastructure, DataImport
VignetteBuilder: knitr
ByteCompile: TRUE
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
SwitchrLibrary: gDRimport
DeploySubPath: gDRimport
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/gDRimport
git_branch: devel
git_last_commit: b9e01bd
git_last_commit_date: 2026-06-17
Repository: Bioconductor 3.24
Date/Publication: 2026-06-17
NeedsCompilation: no
Packaged: 2026-06-17 22:01:00 UTC; biocbuild
Author: Arkadiusz Gladki [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7059-6378>),
  Bartosz Czech [aut] (ORCID: <https://orcid.org/0000-0002-9908-3007>),
  Marc Hafner [aut] (ORCID: <https://orcid.org/0000-0003-1337-7598>),
  Sergiu Mocanu [aut],
  Dariusz Scigocki [aut],
  Allison Vuong [aut],
  Luca Gerosa [aut] (ORCID: <https://orcid.org/0000-0001-6805-9410>),
  Janina Smola [aut]
Maintainer: Arkadiusz Gladki <gladki.arkadiusz@gmail.com>
