global_valid_notes_l <- list(
  list(
    length = 5,
    index_to_check = 2,
    text_to_check = "installed size is"
  ),
  list(
    length = 4,
    index_to_check = 2,
    text_to_check = "installed size is"
  ),
  list(
    length = 5,
    index_to_check = 2,
    text_to_check = "Found the following hidden files and directories"
  )
)
