% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_files.R
\name{load_templates_tsv}
\alias{load_templates_tsv}
\title{Load templates from tsv}
\usage{
load_templates_tsv(template_file, template_filename = NULL)
}
\arguments{
\item{template_file}{character, file path(s) to template(s)}

\item{template_filename}{character, file name(s)}
}
\value{
data.table with template data
}
\description{
This functions loads and checks the template file(s)
}
\keyword{load_files}
