% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_files.R
\name{get_df_from_raw_unedited_EnVision_df}
\alias{get_df_from_raw_unedited_EnVision_df}
\title{Get final results (as a data.table)
from raw unedited EnVision data.table}
\usage{
get_df_from_raw_unedited_EnVision_df(df, n_row, n_col, barcode_col)
}
\arguments{
\item{df}{raw data.table}

\item{n_row}{number of rows}

\item{n_col}{number of columns}

\item{barcode_col}{column number for barcode data}
}
\value{
data.table derived from EnVision data
}
\description{
Get final results (as a data.table)
from raw unedited EnVision data.table
}
\keyword{load_files}
