% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_files.R
\name{load_templates}
\alias{load_templates}
\title{Load templates}
\usage{
load_templates(df_template_files)
}
\arguments{
\item{df_template_files}{data.table, with datapaths and names of results file(s)
or character with file path of templates file(s)}
}
\value{
data.table with templates data
}
\description{
This functions loads and checks the template file(s)
}
\examples{
 td <- get_test_data()
 t_df <- load_templates(template_path(td))

}
\keyword{load_files}
